package pl.qumak.pull;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.builder.SpringApplicationBuilder;
import org.springframework.context.annotation.Configuration;
import org.springframework.ws.client.core.support.WebServiceGatewaySupport;
import pl.qumak.Client;
import pl.qumak.xsd.datex2.ConfidentialityValueEnum;
import pl.qumak.xsd.datex2.CountryEnum;
import pl.qumak.xsd.datex2.D2LogicalModel;
import pl.qumak.xsd.datex2.Exchange;
import pl.qumak.xsd.datex2.HeaderInformation;
import pl.qumak.xsd.datex2.InformationStatusEnum;
import pl.qumak.xsd.datex2.InternationalIdentifier;
import pl.qumak.xsd.datex2.ObjectFactory;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.namespace.QName;
import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.StringWriter;
import java.io.Writer;

@Configuration
public class PullService extends WebServiceGatewaySupport {

    private String companyName;

    public void setCompanyName(String companyName) {
        this.companyName = companyName;
    }


    public void callWebService(SpringApplicationBuilder builder)  {
        Client wsClient = builder.context().getBean(Client.class);
        D2LogicalModel request = createD2();
        D2LogicalRequestToXml(request);
        D2LogicalModel response = wsClient.pullRequest(request);
        D2LogicalResponseToXml(response);
    }

    private D2LogicalModel createD2() {
        ObjectFactory objectFactory = new ObjectFactory();
        D2LogicalModel d2LogicalModel = objectFactory.createD2LogicalModel();
        Exchange exchange = objectFactory.createExchange();
        d2LogicalModel.setModelBaseVersion("2");

        InternationalIdentifier internationalIdentifier = objectFactory.createInternationalIdentifier();
        internationalIdentifier.setCountry(CountryEnum.PL);
        internationalIdentifier.setNationalIdentifier(companyName);
        exchange.setSupplierIdentification(internationalIdentifier);

        d2LogicalModel.setExchange(exchange);
        return d2LogicalModel;
    }

    public void D2LogicalRequestToXml(D2LogicalModel d2LogicalModel){
        Writer writer = null;
        System.out.println("Przygotowano komunikat do wysłania do serwera KPD. Jego treść została zapisana do pliku pullRequest.xml.");
        String pullRequest = marshalXmlToString(d2LogicalModel);
        try {
            writer = new BufferedWriter(new OutputStreamWriter(
                    new FileOutputStream("pullRequest.xml"), "utf-8"));
            writer.write(pullRequest);
        } catch (IOException ex) {
            ex.printStackTrace();
        } finally {
            try {
                writer.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

    }

    private void D2LogicalResponseToXml(D2LogicalModel response) {
        Writer writer = null;
        System.out.println("Odebrano komunikat od serwera KPD. Jego treść została zapisana do pliku pullResponse.xml");
        String pullResponse = marshalXmlToString(response);
        try {
            writer = new BufferedWriter(new OutputStreamWriter(
                    new FileOutputStream("pullResponse.xml"), "utf-8"));
            writer.write(pullResponse);
        } catch (IOException ex) {
            ex.printStackTrace();
        } finally {
            try {
                writer.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public String marshalXmlToString(Object d2logicalModel) {
        String xmlString = null;
        try {
            JAXBContext jaxbContext = JAXBContext.newInstance(D2LogicalModel.class);
            Marshaller marshaller = jaxbContext.createMarshaller();
            marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            StringWriter stringWriter = new StringWriter();
            marshaller.marshal(d2logicalModel, stringWriter);
            xmlString = stringWriter.toString();
        } catch (JAXBException e) {
            e.printStackTrace();
        }
        return xmlString;
    }


}