package pl.qumak.push;

import org.springframework.boot.builder.SpringApplicationBuilder;
import pl.qumak.Client;
import pl.qumak.xsd.datex2.Comment;
import pl.qumak.xsd.datex2.CommentTypeEnum;
import pl.qumak.xsd.datex2.ConfidentialityValueEnum;
import pl.qumak.xsd.datex2.CountryEnum;
import pl.qumak.xsd.datex2.D2LogicalModel;
import pl.qumak.xsd.datex2.Exchange;
import pl.qumak.xsd.datex2.HeaderInformation;
import pl.qumak.xsd.datex2.Impact;
import pl.qumak.xsd.datex2.InformationStatusEnum;
import pl.qumak.xsd.datex2.InternationalIdentifier;
import pl.qumak.xsd.datex2.MultilingualString;
import pl.qumak.xsd.datex2.ObjectFactory;
import pl.qumak.xsd.datex2.OverallPeriod;
import pl.qumak.xsd.datex2.Point;
import pl.qumak.xsd.datex2.PointByCoordinates;
import pl.qumak.xsd.datex2.PointCoordinates;
import pl.qumak.xsd.datex2.ProbabilityOfOccurrenceEnum;
import pl.qumak.xsd.datex2.Situation;
import pl.qumak.xsd.datex2.SituationPublication;
import pl.qumak.xsd.datex2.SituationRecord;
import pl.qumak.xsd.datex2.TrafficConstrictionTypeEnum;
import pl.qumak.xsd.datex2.Validity;
import pl.qumak.xsd.datex2.ValidityStatusEnum;
import pl.qumak.xsd.datex2.WeatherRelatedRoadConditionTypeEnum;
import pl.qumak.xsd.datex2.WeatherRelatedRoadConditions;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.stream.StreamSource;
import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.time.LocalDateTime;

public class PushService {

    private String companyName;

    public void setCompanyName(String companyName) {
        this.companyName = companyName;
    }

    public void callWebService(SpringApplicationBuilder builder) {
        Client wsClient = builder.context().getBean(Client.class);
        D2LogicalModel request = createD2();
        D2LogicalRequestToXml(request);
        D2LogicalModel response = wsClient.pushRequest(request);
        D2LogicalResponseToXML(response);
    }

    private D2LogicalModel createD2() {
        ObjectFactory objectFactory = new ObjectFactory();
        D2LogicalModel d2LogicalModel = objectFactory.createD2LogicalModel();
        Exchange exchange = objectFactory.createExchange();
        d2LogicalModel.setModelBaseVersion("2");
        InternationalIdentifier internationalIdentifier = objectFactory.createInternationalIdentifier();
        internationalIdentifier.setCountry(CountryEnum.PL);
        internationalIdentifier.setNationalIdentifier(companyName);
        exchange.setSupplierIdentification(internationalIdentifier);


        HeaderInformation headerInformation = objectFactory.createHeaderInformation();
        headerInformation.setConfidentiality(ConfidentialityValueEnum.NO_RESTRICTION);
        headerInformation.setInformationStatus(InformationStatusEnum.REAL);

        SituationPublication situationPublication = objectFactory.createSituationPublication();
        situationPublication.setPublicationTime(LocalDateTime.now());
        situationPublication.setPublicationCreator(internationalIdentifier);
        situationPublication.setLang("PL");

        SituationRecord situationRecord = createSituationRecord();
        situationRecord.setId("sr1245");
        situationRecord.setVersion("1");

        situationRecord.setSituationRecordCreationTime(LocalDateTime.now());
        situationRecord.setSituationRecordVersionTime(LocalDateTime.now());
        situationRecord.setProbabilityOfOccurrence(ProbabilityOfOccurrenceEnum.CERTAIN);

        Validity validity = objectFactory.createValidity();
        validity.setValidityStatus(ValidityStatusEnum.ACTIVE);
        OverallPeriod overallPeriod = objectFactory.createOverallPeriod();
        overallPeriod.setOverallStartTime(LocalDateTime.now());
        validity.setValidityTimeSpecification(overallPeriod);
        situationRecord.setValidity(validity);

        Impact impact = objectFactory.createImpact();
        impact.setTrafficConstrictionType(TrafficConstrictionTypeEnum.ROAD_BLOCKED);
        situationRecord.setImpact(impact);

        Comment generalPublicComment = objectFactory.createComment();
        generalPublicComment.setCommentType(CommentTypeEnum.DESCRIPTION);
        StringHelper stringHelper = new StringHelper(objectFactory, CountryEnum.PL);
        MultilingualString comment = stringHelper
                .createFrom("Zablokowany pas lewy w kierunku Łodzi. Ruch odbywa się pasem prawym w kierunku Łodzi.\n");
        generalPublicComment.setComment(comment);
        situationRecord.getGeneralPublicComment().add(generalPublicComment);

        Point groupOfLocation = objectFactory.createPoint();
        PointByCoordinates pointByCoordinates = objectFactory.createPointByCoordinates();
        PointCoordinates pointCoordinates = objectFactory.createPointCoordinates();
        final float longitude = (float) 20.729871;
        final float latitude = (float) 52.163258;
        pointCoordinates.setLatitude(latitude);
        pointCoordinates.setLongitude(longitude);
        pointByCoordinates.setPointCoordinates(pointCoordinates);
        groupOfLocation.setPointByCoordinates(pointByCoordinates);
        situationRecord.setGroupOfLocations(groupOfLocation);

        Situation situation = objectFactory.createSituation();
        situation.setId("s123");
        situation.setVersion("1");
        situation.setSituationVersionTime(LocalDateTime.now());
        situation.getSituationRecord().add(situationRecord);
        situation.setHeaderInformation(headerInformation);
        situationPublication.getSituation().add(situation);

        d2LogicalModel.setExchange(exchange);
        d2LogicalModel.setPayloadPublication(situationPublication);
        return d2LogicalModel;
    }

    private SituationRecord createSituationRecord() {
        ObjectFactory objectFactory = new ObjectFactory();
        String datexClass = "<weatherRelatedRoadConditionType>blackIce</weatherRelatedRoadConditionType>";
        String datexType = getSituationRecordType("<weatherRelatedRoadConditionType>blackIce</weatherRelatedRoadConditionType>");
        WeatherRelatedRoadConditions weatherRelatedRoadConditions = objectFactory.createWeatherRelatedRoadConditions();
        weatherRelatedRoadConditions.getWeatherRelatedRoadConditionType().add(WeatherRelatedRoadConditionTypeEnum.fromValue(datexType));
        return weatherRelatedRoadConditions;
    }

    private String getSituationRecordType(String datexElement) {
        JAXBElement<String> type = null;
        try {
            JAXBContext jaxbContext = JAXBContext.newInstance(String.class);
            Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
            StreamSource xmlSource = new StreamSource(new StringReader(datexElement));
            type = unmarshaller.unmarshal(xmlSource, String.class);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return type.getValue();
    }

    public void D2LogicalRequestToXml(D2LogicalModel request) {
        Writer writer = null;
        System.out.println("Przygotowano komunikat do wysłania do serwera KPD. Jego treść została zapisana do pliku pushRequest.xml.");
        String pushRequest = marshalXmlToString(request);
        try {
            writer = new BufferedWriter(new OutputStreamWriter(
                    new FileOutputStream("pushRequest.xml"), "utf-8"));
            writer.write(pushRequest);
        } catch (IOException ex) {
            ex.printStackTrace();
        } finally {
            try {
                writer.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    private void D2LogicalResponseToXML(D2LogicalModel response) {
        Writer writer = null;
        System.out.println("Odebrano komunikat od serwera KPD. Jego treść została zapisana do pliku pushResponse.xml");
        String pushResponse = marshalXmlToString(response);
        try {
            writer = new BufferedWriter(new OutputStreamWriter(
                    new FileOutputStream("pushResponse.xml"), "utf-8"));
            writer.write(pushResponse);
        } catch (IOException ex) {
            ex.printStackTrace();
        } finally {
            try {
                writer.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public String marshalXmlToString(Object d2logicalModel) {
        String xmlString = null;
        try {
            JAXBContext jaxbContext = JAXBContext.newInstance(D2LogicalModel.class);
            Marshaller marshaller = jaxbContext.createMarshaller();
            marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            StringWriter stringWriter = new StringWriter();
            marshaller.marshal(d2logicalModel, stringWriter);
            xmlString = stringWriter.toString();
        } catch (JAXBException e) {
            e.printStackTrace();
        }
        return xmlString;
    }
}
